/********************************************************************************
 * Copyright (c) 2024 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_REDUCESUMIMPL_H_
#define AIDGE_CPU_OPERATOR_REDUCESUMIMPL_H_

#include <array>
#include <memory>
#include <tuple>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/ReduceSum.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

namespace Aidge {
class ReduceSumImplForward_cpu
    : public Registrable<ReduceSumImplForward_cpu,
                        std::tuple<DataType, DataType>,
                        std::function<void(const std::vector<std::int32_t>&,
                            DimSize_t,
                            const std::vector<DimSize_t>&,
                            const void *,
                            void *)>> {};
class ReduceSumImpl1DBackward_cpu
    : public Registrable<ReduceSumImpl1DBackward_cpu,
                        std::tuple<DataType, DataType>,
                        std::function<void(const std::vector<std::int32_t>&,
                            DimSize_t,
                            const std::vector<DimSize_t>&,
                            const void *,
                            void *)>> {};

class ReduceSumImpl_cpu : public OperatorImpl {
   public:
    ReduceSumImpl_cpu(const ReduceSum_Op& op) : OperatorImpl(op, "cpu") {}

    static std::unique_ptr<ReduceSumImpl_cpu> create(const ReduceSum_Op &op) {
        return std::make_unique<ReduceSumImpl_cpu>(op);
    }

   public:
    void forward() override;
};

namespace {
static Registrar<ReduceSum_Op> registrarReduceSumImpl_cpu("cpu", Aidge::ReduceSumImpl_cpu::create);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_REDUCESUMIMPL_H_ */
